@@ -51,11 +51,12 @@ struct _GFFontEntry {
 	GFFontEntryType type;
 	GFFileEntry files[2];
 	gint face;
-	gchar *name;
-	gchar *version;
-	gchar *familyname;
-	gchar *speciesname;
-	gchar *weight;
+	guchar *name;
+	guchar *version;
+	guchar *familyname;
+	guchar *speciesname;
+	guchar *notice;
+	guchar *weight;
 	gdouble italicangle;
 };
 
