@@ -57,6 +57,10 @@ typedef struct {
 
 typedef struct {
 	guchar *name; /* Font name */
+	guchar *version;
+	guchar *familyname;
+	guchar *speciesname;
+	guchar *psname;
 	guchar *alias; /* PSName of glyphs */
 } GFIAliasData;
 
@@ -71,7 +75,7 @@ typedef struct {
 
 static void gfi_verify_fontmap (GFFontMap *map);
 static void gfi_verify_font_entry (GFFontEntry *e);
-static void gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f);
+static void gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f, gboolean force);
 static void gfi_verify_pfb_file (GFFontEntry *e, GFFileEntry *f);
 static void gfi_verify_ttf_file (GFFontEntry *e, GFFileEntry *f);
 static GFIFileData *gfi_read_afm_file_data (const guchar *name);
@@ -96,6 +100,7 @@ static void gfi_build_font (GFIFontData * fd);
 static void gfi_build_ttf_font (GFIFontData *fd);
 
 static gboolean gfi_font_is_registered (const guchar *name);
+static GFFontEntry *gfi_font_entry_get_by_psname (const guchar *name);
 
 static FILE *gfi_get_output_stream (void);
 static gboolean gfi_ensure_directory (const gchar *path);
@@ -121,9 +126,11 @@ static gboolean gfi_debug = FALSE;
 static gboolean gfi_smart = FALSE;
 static gboolean gfi_recursive = FALSE;
 static gboolean gfi_clean = FALSE;
+static gboolean gfi_notice = FALSE;
 static gboolean gfi_usermap = TRUE;
 static gboolean gfi_dynamicmap = FALSE;
 static gboolean gfi_staticmap = FALSE;
+static gboolean gfi_query_dirs = FALSE;
 static gchar *gfi_target = NULL;
 
 static void add_path (poptContext ctx, enum poptCallbackReason reason, const struct poptOption *opt, const char *arg, void *data);
@@ -137,6 +144,8 @@ static const struct poptOption options[] = {
 	  N_("Search directories recursively for font files"), NULL },
 	{ "clean", 'c', POPT_ARG_NONE, &gfi_clean, 0,
 	  N_("Start from zero fontmap, instead of parsing old one"), NULL },
+	{ "notice", 'n', POPT_ARG_NONE, &gfi_notice, 0,
+	  N_("Include font notice field (Copyright data) into fontmap"), NULL },
 	{ "user", 'u', POPT_ARG_NONE, &gfi_usermap, 0,
 	  N_("Create $HOME/.gnome/fonts/gnome-print.fontmap"), NULL },
 	{ "dynamic", 'd', POPT_ARG_NONE, &gfi_dynamicmap, 0,
@@ -145,9 +154,15 @@ static const struct poptOption options[] = {
 	  N_("Create $DATADIR/gnome/fonts/gnome-print.fontmap"), NULL },
 	{ "target", 't', POPT_ARG_STRING, &gfi_target, 0,
 	  N_("Write output fontmap to specified file (- for stdout)"), NULL },
+	{ "dir", 0, POPT_ARG_NONE, &gfi_query_dirs, 0,
+	  N_("Return the $DATADIR and $SYSCONFDIR used for this gnome-print installation)"), NULL },
 	{ NULL, '\0', POPT_ARG_CALLBACK, &add_path, 0 },
 	{ "aliases", 'a', POPT_ARG_STRING, NULL, 0,
 	  N_("File describing known aliases"), N_("PATH") },
+#if 0
+	{ "default", 'D', POPT_ARG_STRING, NULL, 0,
+	  N_("Specify default font(s) for any locale"), N_("[LOCALE[,LOCALE]:fontname]") },
+#endif
 	POPT_AUTOHELP
 	{ NULL, '\0', 0, NULL, 0 }
 };
@@ -180,12 +195,14 @@ static const guchar *smart_paths[] = {
 	DATADIR "/groff/font",
 	LIBDIR "/groff/font",
         /* X */
-        "/usr/X11R6/lib/X11/fonts"
+        "/usr/X11R6/lib/X11/fonts",
 	/* Misc */
+	"/usr/share/abisource/fonts"
 };
 #define NUM_SMART_PATHS (sizeof (smart_paths) / sizeof (smart_paths[0]))
 
 static GSList *aliaspath_list = NULL;
+static GSList *default_list = NULL;
 
 /* List of GFIAliasData structures */
 static GSList *alias_list = NULL;
@@ -217,15 +234,7 @@ int main (int argc, char ** argv)
 	char ** args;
 	FILE *of;
 	gint i;
-        int result;
-
-	/* Initialize dictionaries */
-
-	goodafm_dict = g_hash_table_new (g_str_hash, g_str_equal);
-	goodpfb_dict = g_hash_table_new (g_str_hash, g_str_equal);
-	goodttf_dict = g_hash_table_new (g_str_hash, g_str_equal);
-	font_dict = g_hash_table_new (g_str_hash, g_str_equal);
-	goodfont_dict = g_hash_table_new (g_str_hash, g_str_equal);
+	int result;
 
 	/* Parse arguments */
 
@@ -250,6 +259,21 @@ int main (int argc, char ** argv)
 
 	setlocale (LC_NUMERIC, "C");
 
+	/* Directories */
+	if (gfi_query_dirs) {
+		fprintf (stderr, "$DATADIR=%s\n", DATADIR);
+		fprintf (stderr, "$SYSCONFDIR=%s\n", SYSCONFDIR);
+		return 0;
+	}
+
+	/* Initialize dictionaries */
+
+	goodafm_dict = g_hash_table_new (g_str_hash, g_str_equal);
+	goodpfb_dict = g_hash_table_new (g_str_hash, g_str_equal);
+	goodttf_dict = g_hash_table_new (g_str_hash, g_str_equal);
+	font_dict = g_hash_table_new (g_str_hash, g_str_equal);
+	goodfont_dict = g_hash_table_new (g_str_hash, g_str_equal);
+
 	/* Step 0: Smart */
 	if (gfi_smart) {
 		uid_t euid;
@@ -352,7 +376,14 @@ int main (int argc, char ** argv)
 	/*
 	 * Now we are ready to process fonts
 	 *
-	 * We start of Type1Aliases
+	 */
+
+	if (gfi_debug) fprintf (stderr, "Building fonts... ");
+	gfi_build_fonts ();
+	if (gfi_debug) fprintf (stderr, "Done\n");
+
+	/*
+	 * Process aliases
 	 *
 	 */
 
@@ -360,15 +391,6 @@ int main (int argc, char ** argv)
 	gfi_process_aliases ();
 	if (gfi_debug) fprintf (stderr, "Done\n");
 
-	/*
-	 * And build fonts from remaining afm/pfb files
-	 *
-	 */
-
-	if (gfi_debug) fprintf (stderr, "Building fonts... ");
-	gfi_build_fonts ();
-	if (gfi_debug) fprintf (stderr, "Done\n");
-
 	if (!goodfont_list) {
 		if (gfi_debug) fprintf (stderr, "NO FONTS\n");
 	} else {
@@ -417,15 +439,24 @@ gfi_verify_font_entry (GFFontEntry *e)
 {
 	switch (e->type) {
 	case GF_FONT_ENTRY_TYPE1:
-		if (e->files[1].path) gfi_verify_afm_file (e, &e->files[1]);
-		gfi_verify_pfb_file (e, &e->files[0]);
 		if (e->files[1].psname && strcmp (e->files[1].psname, e->files[0].psname)) {
 			GFIAliasData *a;
 			/* We are aliased entry */
 			a = g_new (GFIAliasData, 1);
 			a->name = g_strdup (e->name);
+			a->familyname = g_strdup (e->familyname);
+			a->speciesname = g_strdup (e->speciesname);
+			a->psname = g_strdup (e->files[1].psname);
+			a->version = g_strdup (e->version);
 			a->alias = g_strdup (e->files[0].psname);
 			alias_list = g_slist_prepend (alias_list, a);
+			/* Verify pfb as usual */
+			gfi_verify_pfb_file (e, &e->files[0]);
+			/* Forced verify of afm */
+			if (e->files[1].path) gfi_verify_afm_file (e, &e->files[1], TRUE);
+		} else {
+			gfi_verify_pfb_file (e, &e->files[0]);
+			if (e->files[1].path) gfi_verify_afm_file (e, &e->files[1], FALSE);
 		}
 		break;
 	case GF_FONT_ENTRY_TRUETYPE:
@@ -447,7 +478,7 @@ gfi_verify_font_entry (GFFontEntry *e)
  */
 
 static void
-gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f)
+gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f, gboolean forced)
 {
 	GFIFileData *fd;
 
@@ -455,7 +486,7 @@ gfi_verify_afm_file (GFFontEntry *e, GFFileEntry *f)
 	fd = g_hash_table_lookup (goodafm_dict, f->path);
 	if (fd) return;
 
-	if (!gfi_test_file_changed (f)) {
+	if (!forced && !gfi_test_file_changed (f)) {
 		fd = g_new0 (GFIFileData, 1);
 		fd->type = GFI_FILE_AFM;
 		fd->entry.path = g_strdup (f->path);
@@ -920,7 +951,43 @@ gfi_process_aliases (void)
 static void
 gfi_process_alias (GFIAliasData *a)
 {
-	GFFontEntry *new;
+	GFFontEntry *new, *fe;
+
+	/* Return if we are already registered */
+	/* fixme: We should test versions here */
+	if (g_hash_table_lookup (goodfont_dict, a->name)) return;
+
+	/* Check, whether we have original font */
+	fe = gfi_font_entry_get_by_psname (a->alias);
+	if (!fe) return;
+
+	/* Simply compose aliased entry */
+	new = g_new0 (GFFontEntry, 1);
+	new->next = NULL;
+	new->type = GF_FONT_ENTRY_TYPE1;
+	new->name = g_strdup (a->name);
+	new->version = g_strdup (a->version);
+	new->familyname = g_strdup (a->familyname);
+	new->speciesname = g_strdup (a->speciesname);
+	new->notice = NULL;
+	new->weight = g_strdup (fe->weight);
+	new->italicangle = fe->italicangle;
+	/* AFM */
+	new->files[1] = fe->files[1];
+	new->files[1].path = g_strdup (new->files[1].path);
+	/* fixme: This is hackish, but gos for now */
+	new->files[1].psname = g_strdup (a->psname);
+	/* PFB */
+	new->files[0] = fe->files[0];
+	new->files[0].path = g_strdup (new->files[0].path);
+	new->files[0].psname = g_strdup (new->files[0].psname);
+
+	/* Register it */
+	if (gfi_debug) fprintf (stderr, "Registered Type1 alias: %s\n", new->name);
+
+	goodfont_list = g_slist_prepend (goodfont_list, new);
+	g_hash_table_insert (goodfont_dict, new->name, new);
+#if 0
 	GFIFontData *afmfd, *pfbfd;
 	GFIFileData *afm, *pfb;
 	GSList *l;
@@ -934,7 +1001,7 @@ gfi_process_alias (GFIAliasData *a)
 	if (g_hash_table_lookup (goodfont_dict, a->name)) return;
 
 	/* Check for fontdata with afm entry */
-	afmfd = g_hash_table_lookup (font_dict, a->name);
+	afmfd = g_hash_table_lookup (font_dict, a->alias);
 	if (!afmfd) return;
 	if (!afmfd->afm_list) return;
 	afm = afmfd->afm_list->data;
@@ -1008,6 +1075,7 @@ gfi_process_alias (GFIAliasData *a)
 	new->version = g_strdup (afm->version);
 	new->familyname = g_strdup (fi->gfi->familyName);
 	new->speciesname = gfi_get_species_name (new->name, new->familyname);
+	new->notice = (fi->gfi->notice) ? g_strdup (fi->gfi->notice) : NULL;
 	new->weight = g_strdup (fi->gfi->weight);
 	/* AFM */
 	new->files[1] = afm->entry;
@@ -1025,6 +1093,7 @@ gfi_process_alias (GFIAliasData *a)
 
 	goodfont_list = g_slist_prepend (goodfont_list, new);
 	g_hash_table_insert (goodfont_dict, new->name, new);
+#endif
 }
 
 static void
@@ -1103,6 +1172,7 @@ gfi_build_font (GFIFontData *fd)
 	new->version = g_strdup (pfbdata->version);
 	new->familyname = g_strdup (pfb->gfi.familyName);
 	new->speciesname = gfi_get_species_name (new->name, new->familyname);
+	new->notice = (pfb->gfi.notice) ? g_strdup (pfb->gfi.notice) : NULL;
 	new->weight = g_strdup (pfb->gfi.weight);
 	/* PFB */
 	new->files[0] = pfbdata->entry;
@@ -1172,6 +1242,7 @@ gfi_build_ttf_font (GFIFontData *fd)
 	new->version = g_strdup (ttfdata->version);
 	new->familyname = g_strdup (fd->familyname);
 	new->speciesname = gfi_get_species_name (new->name, new->familyname);
+	new->notice = (ttf->gfi.notice) ? g_strdup (ttf->gfi.notice) : NULL;
 	new->weight = g_strdup (ttf->gfi.weight);
 	/* TTF */
 	new->files[0] = ttfdata->entry;
@@ -1221,6 +1292,54 @@ gfi_font_is_registered (const guchar *name)
 	}
 }
 
+static GFFontEntry *
+gfi_font_entry_get_by_psname (const guchar *name)
+{
+	GFFontMap *map;
+	GFFontEntry *fe;
+	GSList *l;
+
+	/* Step 0 - scan goodfont list */
+	for (l = goodfont_list; l != NULL; l = l->next) {
+		fe = (GFFontEntry *) l->data;
+		if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+	}
+	/* Step 1 - other fontmaps of same level */
+	for (map = mastermap->next; map != NULL; map = map->next) {
+		for (fe = map->fonts; fe != NULL; fe = fe->next) {
+			if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+		}
+	}
+	/* Step 2 - next levels */
+	if (gfi_staticmap) {
+		/* Not found in static map, return */
+		return NULL;
+	} else if (gfi_dynamicmap) {
+		/* Search from static maps */
+		for (map = masterdb->staticmaps; map != NULL; map = map->next) {
+			for (fe = map->fonts; fe != NULL; fe = fe->next) {
+				if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+			}
+		}
+		return FALSE;
+	} else {
+		/* Search from dynamic maps */
+		for (map = masterdb->dynamicmaps; map != NULL; map = map->next) {
+			for (fe = map->fonts; fe != NULL; fe = fe->next) {
+				if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+			}
+		}
+		/* Search from static maps */
+		for (map = masterdb->staticmaps; map != NULL; map = map->next) {
+			for (fe = map->fonts; fe != NULL; fe = fe->next) {
+				if ((fe->type == GF_FONT_ENTRY_TYPE1) && (fe->files[1].path) && !strcmp (fe->files[0].psname, name)) return fe;
+			}
+		}
+	}
+
+	return NULL;
+}
+
 static FILE *
 gfi_get_output_stream (void)
 {
@@ -1328,7 +1447,7 @@ gfi_write_font (xmlNodePtr root, GFFontEntry *e)
 		/* Set format */
 		if (e->files[0].psname && e->files[1].psname && strcmp (e->files[0].psname, e->files[1].psname)) {
 			xmlSetProp (n, "format", "type1alias");
-			xmlSetProp (n, "alias", e->files[1].psname);
+			xmlSetProp (n, "alias", e->files[0].psname);
 		} else {
 			xmlSetProp (n, "format", "type1");
 		}
@@ -1379,7 +1498,8 @@ gfi_write_font (xmlNodePtr root, GFFontEntry *e)
 	xmlSetProp (n, "version", e->version);
 	xmlSetProp (n, "familyname", e->familyname);
 	xmlSetProp (n, "speciesname", e->speciesname);
-	xmlSetProp (n, "psname", e->files[0].psname);
+	xmlSetProp (n, "psname", e->files[1].psname ? e->files[1].psname : e->files[0].psname);
+	if (gfi_notice && e->notice) xmlSetProp (n, "notice", e->notice);
 	xmlSetProp (n, "weight", e->weight);
 	g_snprintf (c, 128, "%g", e->italicangle);
 	xmlSetProp (n, "italicangle", c);
@@ -1432,21 +1552,31 @@ static void
 gfi_read_alias (xmlNodePtr node)
 {
 /* Here we should read alias entry and create GFIAliasData */
-	xmlChar *xmlfullname;
+	xmlChar *xmlfullname, *xmlfamilyname, *xmlversion, *xmlname;
 	xmlChar *xmlalias;
 
 	xmlfullname = xmlGetProp (node, "fullname");
+	xmlfamilyname = xmlGetProp (node, "familyname");
+	xmlversion = xmlGetProp (node, "version");
+	xmlname = xmlGetProp (node, "name");
 	xmlalias = xmlGetProp (node, "alias");
 
-	if (xmlfullname && xmlalias) {
+	if (xmlfullname && xmlfamilyname && xmlname && xmlalias) {
 		GFIAliasData *a;
-		a = g_new (GFIAliasData, 1);
+		a = g_new0 (GFIAliasData, 1);
 		a->name = g_strdup (xmlfullname);
+		a->familyname = g_strdup (xmlfamilyname);
+		a->speciesname = gfi_get_species_name (xmlfullname, xmlfamilyname);
+		a->version = (xmlversion) ? g_strdup (xmlversion) : "0.0";
+		a->psname = g_strdup (xmlname);
 		a->alias = g_strdup (xmlalias);
 		alias_list = g_slist_prepend (alias_list, a);
 	}
 
 	if (xmlfullname) xmlFree (xmlfullname);
+	if (xmlfamilyname) xmlFree (xmlfamilyname);
+	if (xmlversion) xmlFree (xmlversion);
+	if (xmlname) xmlFree (xmlname);
 	if (xmlalias) xmlFree (xmlalias);
 }
 
@@ -1457,12 +1587,16 @@ add_path (poptContext ctx, enum poptCallbackReason reason, const struct poptOpti
 {
 	struct stat s;
 
-	if ((stat (arg, &s) == 0) && (S_ISREG (s.st_mode))) {
+	if (arg) {
 		if (opt->shortName == 'a') {
-			aliaspath_list = g_slist_append (aliaspath_list, g_strdup (arg));
+			if ((stat (arg, &s) == 0) && (S_ISREG (s.st_mode)) && arg) {
+				aliaspath_list = g_slist_append (aliaspath_list, g_strdup (arg));
+			} else {
+				if (gfi_debug) fprintf (stderr, "%s is not a regular file\n", arg);
+			}
+		} else if (opt->shortName == 'D') {
+			default_list = g_slist_prepend (default_list, g_strdup (arg));
 		}
-	} else {
-		if (gfi_debug) fprintf (stderr, "%s is not a regular file\n", arg);
 	}
 }
 
