@@ -422,6 +422,7 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, GnomeGlyphList *gl)
 		ps = pgl->strings + s;
 
 		ret += gp_ps2_set_font_private (pc, gnome_rfont_get_font (ps->rfont));
+		g_return_val_if_fail (ps2->selectedfont && ps2->selectedfont->pso, GNOME_PRINT_ERROR_UNKNOWN);
 		ret += gp_ps2_set_color_private (pc,
 						 ((ps->color >> 24) & 0xff) / 255.0,
 						 ((ps->color >> 16) & 0xff) / 255.0,
@@ -729,12 +730,11 @@ gnome_print_ps2_new (GnomePrinter *printer, const char *paper_name)
 	if (!gnome_print_context_open_file (GNOME_PRINT_CONTEXT (ps2), printer->filename)) goto failure;
 
 	paper = gnome_paper_with_name (paper_name);
-	if (!paper) goto failure;
 
 	/* Fill fake bbox */
 	ps2->bbox.x0 = ps2->bbox.y0 = 0.0;
-	ps2->bbox.x1 = gnome_paper_pswidth (paper);
-	ps2->bbox.y1 = gnome_paper_psheight (paper);
+	ps2->bbox.x1 = paper ? gnome_paper_pswidth (paper) : 21 * 72 / 2.54;
+	ps2->bbox.y1 = paper ? gnome_paper_psheight (paper) : 29.7 * 72 / 2.54;
 
 	return ps2;
 
@@ -1040,7 +1040,7 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	/* fixme: %%DocumentData: */
 	/* fixme: Should we use %%Extensions: and drop LanguageLevel? */
 	gnome_print_context_fprintf (pc, "%%%%LanguageLevel: 2" EOL);
-	gnome_print_context_fprintf (pc, "%%%%Pages: %d" EOL, ps2->pages->number);
+	gnome_print_context_fprintf (pc, "%%%%Pages: %d" EOL, ps2->pages ? ps2->pages->number : 0);
 	gnome_print_context_fprintf (pc, "%%%%BoundingBox: %d %d %d %d" EOL,
 				     (gint) floor (ps2->bbox.x0),
 				     (gint) floor (ps2->bbox.y0),
