@@ -126,6 +126,16 @@ gp_fontmap_unref (GPFontMap * map)
 			gp_font_entry_unref ((GPFontEntry *) map->fonts->data);
 			map->fonts = g_slist_remove (map->fonts, map->fonts->data);
 		}
+		while (map->defaults) {
+			GSList *l;
+			l = map->defaults->data;
+			map->defaults = g_slist_remove (map->defaults, l);
+			while (l) {
+				g_free (l->data);
+				l = g_slist_remove (l, l->data);
+			}
+		}
+		if (map->defaultsdict) g_hash_table_destroy (map->defaultsdict);
 		g_free (map);
 	}
 }
@@ -168,6 +178,8 @@ gp_fontmap_load (void)
 	map->families = NULL;
 	map->fontlist = NULL;
 	map->familylist = NULL;
+	map->defaults = NULL;
+	map->defaultsdict = g_hash_table_new (g_str_hash, g_str_equal);
 
 	/* User map */
 	homedir = g_get_home_dir ();
@@ -237,6 +249,60 @@ gp_fontmap_load (void)
 		f->fonts = g_slist_sort (f->fonts, gp_fe_sortspecies);
 	}
 
+	/* Compose defaultsdict */
+	map->defaults = g_slist_reverse (map->defaults);
+	while (map->defaults) {
+		GSList *l;
+		guchar *locales, *fontname;
+		GPFontEntry *entry;
+		l = map->defaults->data;
+		map->defaults = g_slist_remove (map->defaults, l);
+		locales = l->data;
+		fontname = l->next->data;
+		g_slist_free (l);
+		entry = g_hash_table_lookup (map->fontdict, fontname);
+		if (!entry) {
+			GPFamilyEntry *fe;
+			/* Try family */
+			fe = g_hash_table_lookup (map->familydict, fontname);
+			if (fe && fe->fonts) {
+				entry = (GPFontEntry *) fe->fonts->data;
+				l = fe->fonts;
+				while (l) {
+					GPFontEntry *e;
+					e = (GPFontEntry *) l->data;
+					if (!strcasecmp (e->speciesname, "regular") ||
+					    !strcasecmp (e->speciesname, "roman") ||
+					    !strcasecmp (e->speciesname, "normal")) {
+						entry = e;
+						break;
+					}
+					l = l->next;
+				}
+			}
+		}
+		if (entry) {
+			guchar *l;
+			l = locales;
+			while (l) {
+				guchar *e;
+				e = strchr (l, ',');
+				if (e) {
+					*e = '\0';
+					e += 1;
+				}
+				if (!g_hash_table_lookup (map->defaultsdict, l)) {
+					GQuark q;
+					q = g_quark_from_string (l);
+					g_hash_table_insert (map->defaultsdict, g_quark_to_string (q), entry);
+				}
+				l = e;
+			}
+		}
+		g_free (locales);
+		g_free (fontname);
+	}
+
 	return map;
 }
 
@@ -323,131 +389,6 @@ gp_fontmap_load_file (GPFontMap *map, const guchar *filename)
 	}
 }
 
-#if 0
-static void
-gp_fm_load_fonts (GPFontMap * map, xmlDoc * doc)
-{
-	xmlNodePtr root;
-
-	root = xmlDocGetRootElement (doc);
-
-	if (!strcmp (root->name, "fontmap")) {
-		xmlChar * version;
-
-		version = xmlGetProp (root, "version");
-
-		if (!version) {
-			/* Version 1.0 */
-			gp_fm_load_fonts_1_0 (map, root);
-		} else if (!strcmp (version, "2.0")) {
-			/* Version 2.0 */
-			xmlFree (version);
-			gp_fm_load_fonts_2_0 (map, root);
-		} else {
-			xmlFree (version);
-		}
-	}
-}
-
-static void
-gp_fm_load_fonts_1_0 (GPFontMap * map, xmlNodePtr root)
-{
-	xmlNodePtr child;
-
-	child = root->xmlChildrenNode;
-
-	while (child) {
-		xmlChar * format;
-		format = xmlGetProp (child, "format");
-		if (format) {
-			if (!strcmp (format, "type1")) {
-				/* We are type1 entry */
-				gp_fm_load_font_1_0 (map, child);
-			}
-			xmlFree (format);
-		}
-		child = child->next;
-	}
-}
-
-static void
-gp_fm_load_font_1_0 (GPFontMap * map, xmlNodePtr node)
-{
-	GPFontEntryT1 * t1;
-	GPFontEntry * e;
-	gchar * alias, * p;
-
-	alias = gp_xmlGetPropString (node, "alias");
-
-	if (alias) {
-		GPFontEntryT1Alias * t1a;
-		t1a = g_new0 (GPFontEntryT1Alias, 1);
-		t1a->t1.entry.type = GP_FONT_ENTRY_TYPE1_ALIAS;
-		t1a->alias = alias;
-		t1 = (GPFontEntryT1 *) t1a;
-	} else {
-		t1 = g_new0 (GPFontEntryT1, 1);
-		t1->entry.type = GP_FONT_ENTRY_TYPE1;
-	}
-
-	e = (GPFontEntry *) t1;
-
-	e->refcount = 1;
-	e->face = NULL;
-
-	t1->afm.name = gp_xmlGetPropString (node, "metrics");
-	t1->pfb.name = gp_xmlGetPropString (node, "glyphs");
-	e->name = gp_xmlGetPropString (node, "fullname");
-	e->version = gp_xmlGetPropString (node, "version");
-	e->familyname = gp_xmlGetPropString (node, "familyname");
-	e->psname = gp_xmlGetPropString (node, "name");
-
-	if (!(t1->afm.name && t1->pfb.name && e->name && e->familyname && e->psname)) {
-		gp_font_entry_unref (e);
-		return;
-	}
-
-	/* fixme: check integrity */
-	/* fixme: load afm */
-
-	/* Read fontmap 1.0 weight string */
-
-	e->weight = gp_xmlGetPropString (node, "weight");
-	if (e->weight) {
-		t1->Weight = gp_fontmap_lookup_weight (e->weight);
-	} else {
-		e->weight = g_strdup ("Book");
-		t1->Weight = GNOME_FONT_BOOK;
-	}
-
-	/* Discover species name */
-
-	e->speciesname = gp_fm_get_species_name (e->name, e->familyname);
-
-	/* Parse Italic from species name */
-
-	p = strstr (e->speciesname, "Italic");
-	if (!p) p = strstr (e->speciesname, "Oblique");
-
-	if (p) {
-		t1->ItalicAngle = -10.0;
-	} else {
-		t1->ItalicAngle = 0.0;
-	}
-
-	/* fixme: fixme: fixme: */
-
-	if (g_hash_table_lookup (map->fontdict, e->name)) {
-		gp_font_entry_unref (e);
-		return;
-	}
-
-	g_hash_table_insert (map->fontdict, e->name, e);
-	map->num_fonts++;
-	map->fonts = g_slist_prepend (map->fonts, e);
-}
-#endif
-
 /* Parse root element and build fontmap step 1 */
 
 static void
@@ -473,6 +414,22 @@ gp_fm_load_fonts_2_0 (GPFontMap * map, xmlNodePtr root)
 				}
 				xmlFree (format);
 			}
+		} else if (!strcmp (child->name, "default")) {
+			xmlChar *font;
+			font = xmlGetProp (child, "font");
+			if (font) {
+				xmlChar *locales;
+				guchar *loc;
+				GSList *l;
+				locales = xmlGetProp (child, "locales");
+				loc = (locales) ? g_strdup (locales) : g_strdup ("C");
+				/* fixme: This is not nice (Lauris) */
+				l = g_slist_prepend (NULL, g_strdup (font));
+				l = g_slist_prepend (l, loc);
+				map->defaults = g_slist_prepend (map->defaults, l);
+				if (locales) xmlFree (locales);
+				xmlFree (font);
+			}
 		}
 	}
 }
@@ -712,127 +669,56 @@ gp_font_entry_2_0_truetype_load_files (GPFontEntryTT *tt, xmlNodePtr node)
 	}
 }
 
-#if 0
-static void
-gp_fm_load_font_2_0 (GPFontMap * map, xmlNodePtr node, gboolean alias)
+/* This is experimental method (not public anyways) (Lauris) */
+
+GPFontEntry *gp_font_entry_from_files (GPFontMap *map,
+				       const guchar *name, const guchar *family, const guchar *species, gboolean hidden,
+				       const guchar *filename, gint face, const GSList *additional)
 {
-	GPFontEntryT1 * t1;
-	GPFontEntry * e;
-	gchar * p;
-	xmlNodePtr child;
-	xmlChar * t;
+	GPFontEntrySpecial *s;
+	const GSList *l;
+	gchar *p;
 
-	if (alias) {
-		GPFontEntryT1Alias * t1a;
-		t1a = g_new0 (GPFontEntryT1Alias, 1);
-		t1a->t1.entry.type = GP_FONT_ENTRY_TYPE1_ALIAS;
-		t1a->alias = gp_xmlGetPropString (node, "alias");
-		t1 = (GPFontEntryT1 *) t1a;
-	} else {
-		t1 = g_new0 (GPFontEntryT1, 1);
-		t1->entry.type = GP_FONT_ENTRY_TYPE1;
+	g_return_val_if_fail (map != NULL, NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (family != NULL, NULL);
+	g_return_val_if_fail (species != NULL, NULL);
+	g_return_val_if_fail (filename != NULL, NULL);
+
+	if (!hidden && g_hash_table_lookup (map->fontdict, name)) {
+		g_warning ("file %s: line %d: Font with name %s already exists", __FILE__, __LINE__, name);
 	}
 
-	child = node->xmlChildrenNode;
-	while (child) {
-		if (!strcmp (child->name, "file")) {
-			xmlChar * type;
-			type = xmlGetProp (child, "type");
-			if (!strcmp (type, "afm")) {
-				t1->afm.name = gp_xmlGetPropString (child, "path");
-				t = xmlGetProp (child, "size");
-				if (t) t1->afm.size = atoi (t);
-				if (t) xmlFree (t);
-				t = xmlGetProp (child, "mtime");
-				if (t) t1->afm.mtime = atoi (t);
-				if (t) xmlFree (t);
-			} else if (!strcmp (type, "pfb")) {
-				t1->pfb.name = gp_xmlGetPropString (child, "path");
-				t = xmlGetProp (child, "size");
-				if (t) t1->pfb.size = atoi (t);
-				if (t) xmlFree (t);
-				t = xmlGetProp (child, "mtime");
-				if (t) t1->pfb.mtime = atoi (t);
-				if (t) xmlFree (t);
-			}
-			xmlFree (type);
-		}
-		if (t1->afm.name && t1->pfb.name) break;
-		child = child->next;
+	s = g_new0 (GPFontEntrySpecial, 1);
+	s->entry.type = GP_FONT_ENTRY_SPECIAL;
+	s->entry.refcount = 1;
+	s->entry.face = NULL;
+	s->entry.name = g_strdup (name);
+
+	s->entry.version = g_strdup ("0.0");
+	s->entry.familyname = g_strdup (family);
+	s->entry.speciesname = g_strdup (species);
+	s->entry.psname = g_strdup ("Unnamed");
+
+	s->entry.weight = g_strdup ("Book");
+
+	s->file.name = g_strdup (filename);
+
+	for (l = additional; l != NULL; l = l->next) {
+		s->additional = g_slist_prepend (s->additional, g_strdup (l->data));
 	}
+	s->additional = g_slist_reverse (s->additional);
 
-	e = (GPFontEntry *) t1;
+	s->entry.Weight = gp_fontmap_lookup_weight (s->entry.weight);
+	p = strstr (s->entry.speciesname, "Italic");
+	if (!p) p = strstr (s->entry.speciesname, "Oblique");
+	s->entry.ItalicAngle = p ? -10.0 : 0.0;
 
-	e->refcount = 1;
-	e->face = NULL;
+	s->subface = face;
 
-	if (!(t1->afm.name && t1->pfb.name)) {
-		gp_font_entry_unref (e);
-		return;
-	}
-
-	e->name = gp_xmlGetPropString (node, "name");
-	e->version = gp_xmlGetPropString (node, "version");
-	e->familyname = gp_xmlGetPropString (node, "familyname");
-	e->speciesname = gp_xmlGetPropString (node, "speciesname");
-	e->psname = gp_xmlGetPropString (node, "psname");
-
-	if (!(e->name && e->familyname && e->psname)) {
-		gp_font_entry_unref (e);
-		return;
-	}
-
-	/* fixme: check integrity */
-	/* fixme: load afm */
-
-	/* Read fontmap 1.0 weight string */
-
-	e->weight = gp_xmlGetPropString (node, "weight");
-	if (e->weight) {
-		t1->Weight = gp_fontmap_lookup_weight (e->weight);
-	} else {
-		e->weight = g_strdup ("Book");
-		t1->Weight = GNOME_FONT_BOOK;
-	}
-
-	/* Discover species name */
-
-	if (!e->speciesname) e->speciesname = gp_fm_get_species_name (e->name, e->familyname);
-
-	/* Parse Italic */
-
-	t = xmlGetProp (node, "italicangle");
-	if (!t) {
-		p = strstr (e->speciesname, "Italic");
-		if (!p) p = strstr (e->speciesname, "Oblique");
-		if (p) {
-			t1->ItalicAngle = -10.0;
-		} else {
-			t1->ItalicAngle = 0.0;
-		}
-	} else {
-		t1->ItalicAngle = atof (t);
-		xmlFree (t);
-	}
-
-	/* fixme: fixme: fixme: */
-
-	if (g_hash_table_lookup (map->fontdict, e->name)) {
-		gp_font_entry_unref (e);
-		return;
-	}
-
-	g_hash_table_insert (map->fontdict, e->name, e);
-	map->num_fonts++;
-	map->fonts = g_slist_prepend (map->fonts, e);
+	return &s->entry;
 }
 
-static void
-gp_fm_load_aliases (GPFontMap * map, xmlDoc * doc)
-{
-}
-#endif
-
 /*
  * Font Entry stuff
  *
@@ -859,8 +745,9 @@ gp_font_entry_unref (GPFontEntry * entry)
 	g_return_if_fail (entry->refcount < 3);
 
 	if (--entry->refcount < 1) {
-		GPFontEntryT1 * t1;
-		GPFontEntryT1Alias * t1a;
+		GPFontEntryT1 *t1;
+		GPFontEntryT1Alias *t1a;
+		GPFontEntrySpecial *s;
 
 		g_return_if_fail (entry->face == NULL);
 
@@ -882,6 +769,14 @@ gp_font_entry_unref (GPFontEntry * entry)
 			break;
 		case GP_FONT_ENTRY_ALIAS:
 			break;
+		case GP_FONT_ENTRY_SPECIAL:
+			s = (GPFontEntrySpecial *) entry;
+			if (s->file.name) g_free (s->file.name);
+			while (s->additional) {
+				g_free (s->additional->data);
+				s->additional = g_slist_remove (s->additional, s->additional->data);
+			}
+			break;
 		default:
 			g_assert_not_reached ();
 			break;
